@@ -68,6 +68,7 @@ MainWindow::MainWindow(QWidget *parent, QString appletId) :
     m_Bq27x00 = CommonStatic::Bq27x00Present(this);
     qDebug() << "Module bq27x00" << m_Bq27x00;
 
+    m_Pressed = false;
     m_PbCharge = NULL;
     m_Timer = new QTimer(this);
     m_Timer->setInterval(1000);
@@ -145,27 +146,38 @@ void MainWindow::showSettingsDialog()
 void MainWindow::BmePropertyModifiedSlot(QDBusMessage)
 {
 #ifdef Q_WS_MAEMO_5
+    qDebug() << "------------ BmePropertyModifiedSlot ------------";
     QString command = "hal-device bme";
 
     float temperature = 0.0;
     float vcurrent = 0;
     float perc = 0.0;
+    float consumption = 0.0;
 
-    if (CommonStatic::CurrentSettings->PercType == Settings::SourceDefault){
+    if (CommonStatic::CurrentSettings->PercType == Settings::SourceDefault ||
+        CommonStatic::CurrentSettings->PercType == Settings::SourceDrFrost){
+        qDebug() << "Getting temperature from omap34xx";
         temperature = GetValue("/sys/devices/platform/omap34xx_temp/temp1_input");
     }else if (CommonStatic::CurrentSettings->PercType == Settings::SourceBq27200){
+        qDebug() << "Getting data from bq27200";
         temperature = GetValue("/sys/class/power_supply/bq27200-0/temp");
         perc = GetValue("/sys/class/power_supply/bq27200-0/capacity");
         vcurrent = GetValue("/sys/class/power_supply/bq27200-0/voltage_now");
+        //Fix voltage for kernel v47
         if (vcurrent > 9999)
              vcurrent /= 1000;
     }
 
+    //Fix temperature for kernel v47
     if (temperature > 100)
-        temperature = temperature / 10.0;
+        temperature /= 10.0;
     qDebug() << "Battery temperature: " << temperature;
     ui->lblTemp->setText(QString("%1C").arg(QString::number(temperature, 'f', 1)));
 
+    consumption = GetPowerConsumption();
+    qDebug() << "Power consumption: " << consumption << "mA";
+
+    qDebug() << "Getting data from omap34xx";
     QProcess* proc = new QProcess(this);
     proc->start(command, QIODevice::ReadOnly);
     proc->waitForFinished();
@@ -207,90 +219,115 @@ void MainWindow::BmePropertyModifiedSlot(QDBusMessage)
             }
         }
 
-        chargingChanged = m_WasCharging == charging;
+        chargingChanged = m_WasCharging != charging;
+        if (chargingChanged)
+            qDebug() << "Charging state changed";
 
         //When charging use the bq27x00 percentage, if possible
-        if (charging && m_Bq27x00 && !CommonStatic::CurrentSettings->PercType == Settings::SourceBq27200) {
+        if (charging && m_Bq27x00 && CommonStatic::CurrentSettings->PercType != Settings::SourceBq27200) {
+            qDebug() << "Getting percentage from bq27200 (charging)";
             perc = GetValue("/sys/class/power_supply/bq27200-0/capacity");
         }
 
-        if (design != 0 || perc != 0.0){
-            if (perc == 0.0)
-                perc = current / design * 100.0;
+        if (perc == 0.0){
+            if (CommonStatic::CurrentSettings->PercType == Settings::SourceDrFrost){
+                qDebug() << "Getting percentage (dr_frost)";
+                if (vcurrent >= 4050)
+                    perc = 85.0 + (vcurrent - 4050.0) / 12.0;
+                else if (vcurrent >= 3900)
+                    perc = 70.0 + (vcurrent - 3900.0) / 10.0;
+                else if (vcurrent >= 3800)
+                    perc = 50.0 + (vcurrent - 3800.0) / 5.0;
+                else if (vcurrent >= 3660)
+                    perc = 15.0 + (vcurrent - 3660.0) / 4.0;
+                else if (vcurrent >= 3600)
+                    perc = 5.0 + (vcurrent - 3600.0) / 6.0;
+                else
+                    perc = (vcurrent - 3300.0) / 60.0;
+            }else{
+                qDebug() << "Getting percentage (standard)";
+                if (design > 0)
+                    perc = current / design * 100.0;
+                else
+                    perc = 0;
+            }
+        }
 
-            if (m_WasCharging && !charging)
-                m_LastPercentage = perc;
+        //Reset last values if charging state changed:
+        if (chargingChanged){
+            qDebug() << "Resetting last data (chargingChanged)";
+            m_LastPercentage = 0;
+            m_LastPercentageTime = QDateTime::currentDateTime();
+            m_LastRemainingSecs = 0;
+        }
 
-            int remainingSecs = 0;
-            if (m_LastPercentage != 0){
-                if (!charging){
-                    if (perc == m_LastPercentage || perc > m_LastPercentage){
-                        remainingSecs = m_LastRemainingSecs;
-                    }else{
-                        float delta = m_LastPercentage - perc;
-                        if (delta > 0){
-                            int secs = m_LastPercentageTime.secsTo(QDateTime::currentDateTime());
-
-                            remainingSecs = perc / delta * secs;
-                            m_LastRemainingSecs = remainingSecs;
-                        }
-                    }
+        int remainingSecs = 0;
+        if (m_LastPercentage != 0 || consumption != 0){
+            qDebug() << "Calculating time left";
+            if (!charging){
+                if (consumption > 0){
+                    remainingSecs = current / consumption * 3600.0;
+                }else if (perc == m_LastPercentage || perc > m_LastPercentage){
+                    remainingSecs = m_LastRemainingSecs;
                 }else{
-                    if (perc == m_LastPercentage || perc < m_LastPercentage){
-                        remainingSecs = m_LastRemainingSecs;
-                    }else{
-                        float delta = perc - m_LastPercentage;
-                        if (delta > 0){
-                            int secs = m_LastPercentageTime.secsTo(QDateTime::currentDateTime());
+                    float delta = m_LastPercentage - perc;
+                    if (delta > 0){
+                        int secs = m_LastPercentageTime.secsTo(QDateTime::currentDateTime());
 
-                            remainingSecs = (100 - perc) / delta * secs;
-                            m_LastRemainingSecs = remainingSecs;
-                        }
+                        remainingSecs = perc / delta * (float)secs;
+                        m_LastRemainingSecs = remainingSecs;
                     }
                 }
-            }
-            qDebug() << "Last battery percentage: " << m_LastPercentage;
-            qDebug() << "Battery percentage: " << perc;
-            qDebug() << "Battery charge: " << current << "/" << design;
-            qDebug() << "Battery voltage: " << vcurrent << "/" << vdesign;
-            qDebug() << "Battery time left: " << CommonStatic::FormatHourMinutes(remainingSecs);
-            qDebug() << "Battery is charging: " << charging;
-
-            m_WasCharging = charging;
-
-            if (!charging){
-                if (m_PbCharge)
-                    m_PbCharge->setValue(perc * 100);
-                ui->lblPerc->setText(QString("%1%").arg(QString::number(perc, 'f', 1)));
-            }
-
-            if (chargingChanged)
-                ProgressBarValueSlot(perc * 100);
-            ui->lblCharge->setText(QString("%1 mAh").arg((int)current));
-            ui->lblVoltage->setText(QString("%1 mV").arg((int)vcurrent));
-            ui->lblRemaining->setText(CommonStatic::FormatHourMinutes(remainingSecs));
-
-            if (!charging){
-                m_Timer->stop();
-                if (perc < m_LastPercentage || m_LastPercentage == 0){
-                    m_LastPercentageTime = QDateTime::currentDateTime();
-                    m_LastPercentage = perc;
-                }
             }else{
-                if (perc > m_LastPercentage || m_LastPercentage == 0){
-                    m_LastPercentageTime = QDateTime::currentDateTime();
-                    m_LastPercentage = perc;
+                if (consumption < 0){
+                    remainingSecs = (design - current) / (consumption * -1.0) * 3600.0;
+                }else if (perc == m_LastPercentage || perc < m_LastPercentage){
+                    remainingSecs = m_LastRemainingSecs;
+                }else{
+                    float delta = perc - m_LastPercentage;
+                    if (delta > 0){
+                        int secs = m_LastPercentageTime.secsTo(QDateTime::currentDateTime());
+
+                        remainingSecs = (100.0 - perc) / delta * (float)secs;
+                        m_LastRemainingSecs = remainingSecs;
+                    }
                 }
-                if(!m_Timer->isActive())
-                    m_Timer->start();
+            }
+        }
+        qDebug() << "Last battery percentage: " << m_LastPercentage;
+        qDebug() << "Battery percentage: " << perc;
+        qDebug() << "Battery charge: " << current << "/" << design;
+        qDebug() << "Battery voltage: " << vcurrent << "/" << vdesign;
+        qDebug() << "Battery time left: " << CommonStatic::FormatHourMinutes(remainingSecs);
+        qDebug() << "Battery is charging: " << charging;
+
+        m_WasCharging = charging;
+
+        if (!charging){
+            if (m_PbCharge)
+                m_PbCharge->setValue(perc * 100);
+            ui->lblPerc->setText(QString("%1%").arg(QString::number(perc, 'f', 1)));
+        }
+
+        if (chargingChanged)
+            ProgressBarValueSlot(perc * 100);
+        ui->lblCharge->setText(QString("%1 mAh").arg((int)current));
+        ui->lblVoltage->setText(QString("%1 mV").arg((int)vcurrent));
+        ui->lblRemaining->setText(CommonStatic::FormatHourMinutes(remainingSecs));
+
+        if (!charging){
+            m_Timer->stop();
+            if (perc < m_LastPercentage || m_LastPercentage == 0){
+                m_LastPercentageTime = QDateTime::currentDateTime();
+                m_LastPercentage = perc;
             }
         }else{
-            if (m_PbCharge)
-                m_PbCharge->setValue(0);
-            ui->lblPerc->setText(QString("%1%").arg(QString::number(0, 'f', 1)));
-            ui->lblCharge->setText(QString("%1 mAh").arg(0));
-            ui->lblVoltage->setText(QString("%1 mV").arg(0));
-            ui->lblRemaining->setText("-");
+            if (perc > m_LastPercentage || m_LastPercentage == 0){
+                m_LastPercentageTime = QDateTime::currentDateTime();
+                m_LastPercentage = perc;
+            }
+            if(!m_Timer->isActive())
+                m_Timer->start();
         }
     }else{
         qWarning() << "Command returned" << proc->exitCode();
@@ -489,3 +526,52 @@ void MainWindow::DisplayStateChangedSlot(QString displayState)
             BmePropertyModifiedSlot(QDBusMessage());
     }
 }
+
+float MainWindow::GetPowerConsumption()
+{
+    float res = 0.0;
+
+    //From bq27200:
+    if (QFile::exists("/sys/class/power_supply/bq27200-0/current_now")){
+        qDebug() << "Getting power consumption from bq27200";
+        float curr = GetValue("/sys/class/power_supply/bq27200-0/current_now");
+        res =  curr * 3570.0 / 22.0 / 1000.0;
+    }else{
+        //With i2cget:
+        qDebug() << "Getting power consumption with i2cget";
+        QProcess* proc = new QProcess(this);
+        proc->start("/usr/sbin/i2cget -y 2 0x55 0x14 w", QIODevice::ReadOnly);
+        proc->waitForFinished();
+        if (proc->exitCode() == 0){
+            QByteArray ba = proc->readAllStandardOutput();
+            QString output(ba);
+            output = output.trimmed();
+
+            qDebug() << "i2cget output: " << output;
+            bool ok = false;
+            res = output.toInt(&ok,16);
+            if (ok)
+                res = res * 3570.0 / 22.0 / 1000.0;
+        }
+        delete proc;
+    }
+    return res;
+}
+
+void MainWindow::mousePressEvent(QMouseEvent* event)
+{
+    qDebug() << "mousePressEvent";
+    m_Pressed = true;
+    QMainWindow::mousePressEvent(event);
+}
+
+void MainWindow::mouseReleaseEvent(QMouseEvent* event)
+{
+    qDebug() << "mouseReleaseEvent";
+    if (m_Pressed){
+        qDebug() << "Updating data (user request)";
+        m_Pressed = false;
+        BmePropertyModifiedSlot(QDBusMessage());
+    }
+    QMainWindow::mouseReleaseEvent(event);
+}
