@@ -46,6 +46,7 @@ public:
 private:    
     Ui::MainWindow *ui;
 
+    bool m_Pressed;
     QProgressBar* m_PbCharge;
 
     QTimer* m_Timer;
@@ -61,10 +62,14 @@ private:
     void ApplySettings();
     float GetValue(QString file);
     void SizeToFit();
+    float GetPowerConsumption();
 
 protected:
     void paintEvent(QPaintEvent *event);
     void closeEvent(QCloseEvent *event);
+    void mousePressEvent(QMouseEvent* event);
+    void mouseReleaseEvent(QMouseEvent* event);
+
 public slots:
     void showSettingsDialog();
     void BmePropertyModifiedSlot(QDBusMessage);
