@@ -31,10 +31,10 @@ SettingsDialog::SettingsDialog(QWidget *parent) :
     //setAttribute(Qt::WA_Maemo5AutoOrientation, true);
 #endif
 
-    ui->cmbChargeType->setEnabled(CommonStatic::Bq27x00Present(this));
-
     ui->cmbChargeType->addItem(tr("Default"), Settings::SourceDefault);
-    ui->cmbChargeType->addItem(tr("bq27200"), Settings::SourceBq27200);
+    if (CommonStatic::Bq27x00Present(this))
+        ui->cmbChargeType->addItem(tr("bq27200"), Settings::SourceBq27200);
+    ui->cmbChargeType->addItem(tr("dr_frost_dk"), Settings::SourceDrFrost);
 
     ui->cmbStyle->addItem(tr("Graphical"), Settings::StyleGraphical);
     ui->cmbStyle->addItem(tr("Standard"), Settings::StyleStandard);
@@ -42,7 +42,12 @@ SettingsDialog::SettingsDialog(QWidget *parent) :
 
     connect(ui->btnSalva, SIGNAL(clicked()), this, SLOT(accept()));
 
-    ui->cmbChargeType->setCurrentIndex(CommonStatic::CurrentSettings->PercType);
+    int idx = ui->cmbChargeType->findData(CommonStatic::CurrentSettings->PercType);
+    if (idx >= 0)
+        ui->cmbChargeType->setCurrentIndex(idx);
+    else
+        ui->cmbChargeType->setCurrentIndex(0);
+
     ui->cmbStyle->setCurrentIndex(CommonStatic::CurrentSettings->PercStyle);
     connect(ui->cmbStyle, SIGNAL(currentIndexChanged(int)), this, SLOT(StyleChangedSlot(int)));
     ui->chkLabels->setChecked(CommonStatic::CurrentSettings->ShowLabels);
@@ -65,7 +70,7 @@ SettingsDialog::~SettingsDialog()
 
 void SettingsDialog::accept()
 {
-    CommonStatic::CurrentSettings->PercType = (Settings::PercSource)ui->cmbChargeType->currentIndex();
+    CommonStatic::CurrentSettings->PercType = (Settings::PercSource)ui->cmbChargeType->itemData(ui->cmbChargeType->currentIndex()).toInt();
     CommonStatic::CurrentSettings->PercStyle = (Settings::PercStyles)ui->cmbStyle->currentIndex();
     CommonStatic::CurrentSettings->ShowLabels = ui->chkLabels->isChecked();
     CommonStatic::CurrentSettings->ShowCharge = ui->chkCharge->isChecked();
