@@ -38,6 +38,8 @@ private:
 private slots:
     void accept();
     void OpacityChangedSlot(int value);
+    void ScaleChangedSlot(int value);
+    void StyleChangedSlot(int style);
     void AboutSlot();
 };
 
